//  Created by Robin Speijer on 19-05-17.
//  Copyright © 2017 Bohemian Coding. 

/// The error domain for all errors that are generated by SketchCloudKit.
static NSString * const _Nonnull SCKErrorDomain = @"com.bohemiancoding.cloudkit.error";
static NSString * const _Nonnull SCKKeychainErrorDomain = @"com.bohemiancoding.cloudkit.keychain.error";
static NSString * const _Nonnull SCKAPIEnvironmentErrorKey = @"environment";

static NSInteger const SCKErrorCodeNoContent = 204;
static NSInteger const SCKErrorCodeInvalidFormat = 601;
static NSInteger const SCKErrorCodeInvalidContent = 602;
static NSInteger const SCKErrorCodeInvalidRequest = 400;
static NSInteger const SCKErrorCodeResponseNotMockable = 701;
static NSInteger const SCKErrorCodeDataNotMockable = 702;

@interface NSError (Cloud)

/// Checks if the given response + data represents a server error. If this is the case, the error is constructed and returned.
+ (nullable NSError *)errorFromAPIResponse:(nullable NSURLResponse *)response data:(nullable NSData *)data;

/// Whether this is a Sketch Cloud related error.
@property (nonatomic, readonly) BOOL isCloudError;

/// Whether this is a Cloud error that has been thrown because the requester is not authorized to access the resource.
@property (nonatomic, readonly) BOOL isCloudUnauthorized;

/// Whether this is a Cloud error that has been thrown because the requested resource is forbidden to access.
@property (nonatomic, readonly) BOOL isCloudForbidden;

/// Whether the error is a result of cancelling the original URL request.
@property (nonatomic, readonly) BOOL isUrlCancelled;

/// A Cloud API error could theoretically contain child errors with additional information about the error. This is usually meant for HTTP forms, and separate messages are required for separate fields of information.
@property (nonatomic, nullable, readonly) NSArray<NSDictionary *> *fields;

@end
